// ---------------------------------------------------------------------------
//     LCD HD44780
//
// LCD_Init   LCD
// LCD_Cmd   
// LCD_Char    ASCII   LCD
// LCD_Clear   LCD      
// LCD_Home     LCD   
// LCD_Goto     (x,y)
// LCD_Line      (x)
// LCD_Hex   
// LCD_Integer   
// LCD_String  
#include "settings.h"
#include "misc.h"

//    LCD  6   GPIO: 2  
//   4   .
//  B       HD44780 LCD.
//   ,      AVR
//   LCD:
#define LCD_RS 0  //   LCD R/S (PB0)
#define LCD_E 1   //     LCD
#define DAT4 2    //   d4
#define DAT5 3    //   d5
#define DAT6 4    //   d6
#define DAT7 5    //   d7

//       HD44780.
#define CLEARDISPLAY 0x01
#define SETCURSOR    0x80

void PulseEnableLine ()
//   . 1      LCD.
{
   SetBit(PORTB,LCD_E);    // LCD enable   . 1
   _delay_us(40);          //  40 
   ClearBit(PORTB,LCD_E);  // LCD enable   . 0
}

void SendNibble(byte data)
//      .
{
   PORTB &= 0xC3;       // 1100.0011 =  4  
   if (data & _BV(4)) SetBit(PORTB,DAT4);
   if (data & _BV(5)) SetBit(PORTB,DAT5);
   if (data & _BV(6)) SetBit(PORTB,DAT6);
   if (data & _BV(7)) SetBit(PORTB,DAT7);
   PulseEnableLine();   //    4   
}

void SendByte (byte data)
//      .
{
   SendNibble(data);    //   4 
   SendNibble(data<<4); //   4 
   ClearBit(PORTB,5);   //   LED
}

void LCD_Cmd (byte cmd)
//     LCD.
{
   ClearBit(PORTB,LCD_RS); //  R/S = 0,    
   SendByte(cmd);          //  
}

void LCD_Char (byte ch)
//     LCD  .
{
   SetBit(PORTB,LCD_RS);   //  R/S = 1,    
   SendByte(ch);           //  
}

void LCD_Init()
//    .
{
   LCD_Cmd(0x33);    //   LCD
   LCD_Cmd(0x32);    //    4-   
   LCD_Cmd(0x28);    // 2 ,   5x7
   LCD_Cmd(0x0C);    //   (0x0E  )
   LCD_Cmd(0x06);    //  : 
   LCD_Cmd(0x01);    //  
   msDelay(3);       //    LCD
}

void LCD_Clear()
//   .
{
   LCD_Cmd(CLEARDISPLAY);
   msDelay(3);       //     LCD
}

void LCD_Home()
//    LCD  "" 
// (  ).
{
   LCD_Cmd(SETCURSOR);
}

void LCD_Goto(byte x, byte y)
//    LCD    .
{
   byte addr = 0;                //  0    0x00
   switch (y)
   {
   case 1: addr = 0x40; break;   //  1    0x40
   case 2: addr = 0x14; break;
   case 3: addr = 0x54; break;
   }
   LCD_Cmd(SETCURSOR+addr+x);    //   
}

void LCD_Line(byte row)
//    LCD     .
{
   LCD_Goto(0,row);
}

void LCD_String(const char *text)
//      LCD   .
{
   while (*text)           // ,     /0
      LCD_Char(*text++);   //     
}

void LCD_Hex(int data)
//   HEX-    LCD
//    .
{
   char st[8] = "";        //    
   itoa(data,st,16);       //    hex- ASCII
   //LCD_Message("0x");      //  ,    "0x"
   LCD_String(st);         //    LCD
}

void LCD_Integer(int data)
//       LCD
//    .
{
   char st[8] = "";        //    
   itoa(data,st,10);       //     ASCII
   LCD_String(st);         //    LCD
}
